from itertools import count
import random
import string
from time import time

import gevent
from gevent.event import Event
try:
    import simplejson as json
except ImportError:
    import json

from mushroom.rpc import Engine as RpcEngine


POLL_TIMEOUT = 40
SESSION_CLEANUP_INTERVAL = 30
SESSION_MAX_AGE = 60


def session_id_generator():
    alpha_numerics = string.ascii_letters + string.digits
    while True:
        '''A UUID has 16 bytes and therefore 256^16 possibilities. For the
        shortest possible id which consists only of URL safe characters we
        pick only alpha numeric characters (2*26+10) and make it 22
        characters long. This is at least as good as a hex encoded UUID
        which would require 32 characters.'''
        yield ''.join(random.choice(alpha_numerics) for _ in xrange(22))


class Session(object):

    def __init__(self, id, transport, rpc_handler):
        self.id = id
        self.transport = transport
        self.rpc_engine = RpcEngine(transport, rpc_handler)

    def notify(self, *args, **kwargs):
        '''
        Send a notification to the connected client.

        This method is just a wrapper for the
        :func:`mushroom.rpc.Engine.request` method and uses the
        same arguments.
        '''
        self.rpc_engine.notify(*args, **kwargs)

    def request(self, *args, **kwargs):
        '''
        Send a request to the connected client.

        This method is just a wrapper for the
        :func:`mushroom.rpc.Engine.notify` method and uses the
        same arguments.
        '''
        return self.rpc_engine.request(*args, **kwargs)


class SessionList(object):
    '''
    List of sessions which provides a convenient `notify()` method to
    notify all sessions. This list also implements copy-on-write (COW)
    so calls to `add()` and `remove()` are possible during a `notify()` call.
    '''

    def __init__(self):
        self.sessions = {}

    def add(self, session):
        if session.id in self.sessions:
            raise KeyError('Duplicate session id %r' % session.id)
        self.sessions = self.sessions.copy()
        self.sessions[session.id] = session

    def remove(self, session):
        self.sessions = self.sessions.copy()
        del self.sessions[session.id]

    def notify(self, method, data=None):
        for session in self.sessions.itervalues():
            session.notify(method, data)

    def __getitem__(self, sid):
        return self.sessions[sid]

    def __iter__(self):
        return self.sessions.itervalues()

    def __len__(self):
        return len(self.sessions)


class SessionHandler(object):

    def authenticate(self, session, auth):
        return True

    def connect(self, session):
        pass

    def disconnect(self, session):
        pass


class SessionHandlerAdapter(object):

    def __init__(self, obj, prefix='session_', suffix=''):
        self.obj = obj
        self.prefix = prefix
        self.suffix = suffix

    def __getattr__(self, name):
        return getattr(self.obj, self.prefix + name + self.suffix)



class Signal(object):

    def __init__(self):
        self.handlers = []

    def send(self, *args, **kwargs):
        for handler in self.handlers:
            handler(*args, **kwargs)

    def connect(self, handler):
        self.handlers.append(handler)

    def disconnect(self, handler):
        self.handlers.remove(handler)

    def disconnect_all(self):
        del self.handlers[:]


class Observable(object):

    def __init__(self, value=None):
        self.value = value
        self.listeners = []

    def __call__(self, *args):
        if len(args) == 0:
            return self.get()
        elif len(args) == 1:
            return self.set(args[0])
        else:
            raise TypeError('Observable.__init__ takes zero or one arguments (%d given)' % len(args))

    def set(self, value):
        self.value = value
        for listener in self.listeners:
            listener(value)

    def get(self):
        return self.value

    def subscribe(self, listener):
        self.listeners.append(listener)

    def unsubscribe(self, listener):
        self.listeners.remove(listener)

    def unsubscribe_all(self, listener):
        del self.listeners[:]



#!/usr/bin/env python

from time import time
from random import random

import gevent

import example_pythonpath
from example_utils import ExampleServer, ExampleStaticFile


class TimePusherServer(ExampleServer):
    urls = [
        ('/', ExampleStaticFile('click-game.html')),
    ] + ExampleServer.urls

    def server_init(self):
        self.score = 0
        gevent.spawn(self.main_loop)

    def main_loop(self):
        while True:
            gevent.sleep(2)
            x = random()
            y = random()
            self.sessions.notify('target', { 'x': x, 'y': y })

    def rpc_click(self, request):
        self.score += 1
        self.sessions.notify('score', self.score)


if __name__ == '__main__':
    TimePusherServer.main()



import os
import sys

from mushroom.simple import SimpleServer, StaticFile


EXAMPLES_DIR = os.path.dirname(os.path.abspath(__file__))
PROJECT_ROOT = os.path.dirname(EXAMPLES_DIR)


class ExampleStaticFile(StaticFile):
    def __init__(self, filename):
        super(ExampleStaticFile, self).__init__(
                os.path.join(EXAMPLES_DIR, filename))


class ExampleServer(SimpleServer):
    urls = [
        ('/js/mushroom.js', StaticFile(os.path.join(PROJECT_ROOT, 'js', 'mushroom.js'))),
        ('/js/jquery.js', ExampleStaticFile('jquery-1.8.2.min.js')),
        ('/js/knockout.js', ExampleStaticFile('knockout-2.2.0.min.js')),
        ('/js/gauge.js', ExampleStaticFile('gauge-1.2.min.js')),
    ]



import codecs
import os

import mushroom
from mushroom.http import HttpResponse
from mushroom.server import Server


TESTS_DIR = os.path.dirname(os.path.abspath(__file__))
SELENIUM_JS = os.path.join(TESTS_DIR, 'selenium.js')
PROJECT_ROOT = os.path.dirname(os.path.dirname(TESTS_DIR))
MUSHROOM_JS = os.path.join(PROJECT_ROOT, 'js', 'mushroom.js')


class TestServer(Server):

    def __init__(self, listener, index_template, rpc_handler=None,
            session_handler=None, **kwargs):
        application = TestApplication(
                index_template, rpc_handler, session_handler)
        super(TestServer, self).__init__(listener,
                application,
                **kwargs)


class TestApplication(mushroom.Application):

    def __init__(self, index_template, rpc_handler=None,
            session_handler=None):
        self.index_template = index_template
        super(TestApplication, self).__init__(rpc_handler,
                session_handler)

    def load_file(self, filename):
        with codecs.open(filename) as fh:
            return fh.read()

    def request(self, request):
        if request.method == 'GET':
            if request.path == ['favicon.ico']:
                return HttpResponse('404 Not Found')
            if request.path == ['js', 'mushroom.js']:
                return HttpResponse('200 OK', self.load_file(MUSHROOM_JS),
                        extra_headers={'Content-Type': 'application/javascript'})
            if request.path == ['js', 'selenium.js']:
                return HttpResponse('200 OK', self.load_file(SELENIUM_JS),
                        extra_headers={'Content-Type': 'application/javascript'})
            if request.path == []:
                return HttpResponse('200 OK', self.load_file(self.index_template),
                        extra_headers={'Content-Type': 'text/html'})
        return super(TestApplication, self).request(request)




from django.db import close_old_connections

from mushroom.rpc.middleware import Middleware


class CloseOldConnectionsMiddleware(Middleware):

    def __call__(self, request):
        try:
            return self.get_response(request)
        finally:
            close_old_connections()



import logging
import unittest

from mushroom.utils import Observable
from mushroom.utils import Signal


class NS(object):
    pass


class ObservableTestCase(unittest.TestCase):

    def test_set_and_get(self):
        o = Observable()
        value = object()
        self.assertIs(o(), None)
        o(value)
        self.assertIs(o(), value)

    def test_subscribe(self):
        o = Observable()
        ns = NS()
        ns.success = False
        ns.value = None
        def cb(value):
            ns.success = True
            ns.value = value
        o.subscribe(cb)
        value = object()
        self.assertIs(o(), None)
        o(value)
        self.assertTrue(ns.success)
        self.assertIs(ns.value, value)
        self.assertIs(o(), value)


class SignalTestCase(unittest.TestCase):

    def test_connect(self):
        s = Signal()
        ns = NS()
        ns.success = False
        ns.value = None
        def cb(value):
            ns.success = True
            ns.value = value
        s.connect(cb)
        value = object()
        s.send(value)
        self.assertTrue(ns.success)
        self.assertIs(ns.value, value)

    def test_disconnect(self):
        s = Signal()
        ns = NS()
        ns.success = False
        ns.value = None
        def cb(value):
            ns.success = True
            ns.value = value
        s.connect(cb)
        s.disconnect(cb)
        value = object()
        s.send(value)
        self.assertFalse(ns.success)
        self.assertIs(ns.value, None)



#!/usr/bin/env python

from __future__ import print_function

import cmd
import logging

from gevent import monkey

import example_pythonpath
from example_utils import ExampleServer, ExampleStaticFile


class RemoteControlCmd(cmd.Cmd):

    prompt = 'remote-control> '
    intro = 'Interactive browser remote control\nType "help" for more information.'
    use_rawinput = False

    def __init__(self, server):
        cmd.Cmd.__init__(self)
        self.server = server

    def postcmd(self, stop, line):
        if stop == 'EOF':
            print('^D')
        if stop:
            print('May the maltron be with you!')
            return True

    def do_help(self, args):
        '''Type "help" for the list of available commands and help <command>" for details about a specific command.'''
        cmd.Cmd.do_help(self, args)

    def do_exit(self, args):
        '''Exit the console.'''
        return 'exit'

    def do_eval(self, args):
        '''Call eval() on the browser.'''
        self.server.sessions.notify('eval', args)

    def do_print(self, args):
        '''Call print() on the browser.'''
        self.server.sessions.notify('print', args)

    def do_alert(self, args):
        '''Call alert() on the browser.'''
        self.server.sessions.notify('alert', args)

    def do_EOF(self, args):
        '''You can exit the console by typing Ctrl+D.'''
        return 'EOF'

    def do_who(self, args):
        '''Show connected users'''
        if not self.server.sessions:
            print('No sessions connected.')
            return
        print('%d session%s connected:' % (
            len(self.server.sessions),
            's' if len(self.server.sessions) != 1 else ''))
        print('SESSION ID              IP ADDRESS       TRANSPORT')
        print('--------------------------------------------------')
        #####('xxxxxxxxxxxxxxxxxxxxxx  000.000.000.000  ws/poll  ')
        for session in self.server.sessions:
            print('%-22s  %-15s  %-9s' % (session.id,
                session.transport.remote_addr, session.transport.name))

    def do_gauge(self, args):
        '''Set gauge value'''
        self.server.sessions.notify('gauge', args)


class RemoteControlServer(ExampleServer):
    urls = [
        ('/', ExampleStaticFile('remote-control.html')),
    ] + ExampleServer.urls

    def __init__(self, listener):
        super(RemoteControlServer, self).__init__(listener, log=None)


if __name__ == '__main__':
    monkey.patch_sys()
    logging.basicConfig(filename='remote-control.log', level=logging.DEBUG)
    listener = (RemoteControlServer.host, RemoteControlServer.port)
    print('Server running at http://%s:%d/' % listener)
    server = RemoteControlServer(listener)
    server.start()
    rccmd = RemoteControlCmd(server)
    rccmd.cmdloop()
    # XXX how to shutdown cleanly?



from gevent.pywsgi import WSGIHandler
from geventwebsocket.handler import WebSocketHandler
from gunicorn.workers.ggevent import GeventPyWSGIWorker


class MushroomWebSocketHandler(WebSocketHandler):
    # FIXME implement a custom WebSocketHandler which does not accept any
    # websocket connection but only those with a valid session.
    pass


class MushroomWorker(GeventPyWSGIWorker):
    wsgi_handler = MushroomWebSocketHandler



import logging
import unittest

import gevent
from gevent.event import Event
from gevent import monkey
from gevent.pywsgi import WSGIServer
from kombu import BrokerConnection
from kombu import Exchange, Queue
from kombu import pools

import mushroom
from mushroom import messaging
from mushroom import rpc


SERVER_HOST = '127.0.0.1'
SERVER_PORT = 39288 # picked by random.randint(1024, 65535)

BROKER_URL = 'amqp://mushroom:mushroom@localhost//mushroom'

# During tests the time until a message arrives should never exceed
# one second.
TIMEOUT = 1


class MessagingTestService(object):

    def __init__(self):
        self.success = False
        self.test_done = Event()

    def rpc_ping(self, request):
        self.success = True
        self.test_done.set()

    def rpc_get_answer(self, request):
        return 42

    def rpc_get_fail(self, request):
        raise rpc.RequestException('FAIL!')


class MessagingTestCase(unittest.TestCase):

    def setUp(self):
        logging.basicConfig()
        monkey.patch_socket()
        # prepare exchange and queue
        self.exchange = Exchange('test', type='direct')
        self.monolog_queue = Queue('monolog', self.exchange, routing_key='monolog')
        self.a_queue = Queue('a', self.exchange, routing_key='a')
        self.b_queue = Queue('b', self.exchange, routing_key='b')
        # purge queue so we start with a fresh test
        self.purge(self.monolog_queue, self.a_queue, self.b_queue)

    def tearDown(self):
        self.purge(self.monolog_queue, self.a_queue, self.b_queue)

    def purge(self, *queues):
        self.connection_pool = pools.connections[BrokerConnection(BROKER_URL)]
        with self.connection_pool.acquire() as connection:
            for queue in queues:
                queue(connection.channel()).declare()
                with connection.Consumer(queue) as consumer:
                    consumer.purge()

    def test_rpc_notify(self):
        test_service = MessagingTestService()
        transport = messaging.Transport(BROKER_URL, self.exchange, self.monolog_queue)
        engine = rpc.Engine(transport, mushroom.MethodDispatcher(test_service, prefix='rpc_'))
        try:
            transport.start()
            engine.notify('ping', routing_key='monolog')
            if not test_service.test_done.wait(1):
                self.fail('Timeout')
            self.assertTrue(test_service.success)
        finally:
            transport.stop()

    def test_rpc_request(self):
        test_service = MessagingTestService()
        transport = messaging.Transport(BROKER_URL, self.exchange, self.monolog_queue)
        engine = rpc.Engine(transport, mushroom.MethodDispatcher(test_service, prefix='rpc_'))
        try:
            transport.start()
            answer = engine.request('get_answer', routing_key='monolog', timeout=TIMEOUT)
            self.assertEqual(answer, 42)
        finally:
            transport.stop()

    def test_rpc_request_error(self):
        test_service = MessagingTestService()
        transport = messaging.Transport(BROKER_URL, self.exchange, self.monolog_queue)
        engine = rpc.Engine(transport, mushroom.MethodDispatcher(test_service, prefix='rpc_'))
        try:
            transport.start()
            self.assertRaises(rpc.RequestException, engine.request,
                    'get_fail', routing_key='monolog', timeout=TIMEOUT)
        finally:
            transport.stop()



import logging
import unittest

import gevent
from gevent.event import AsyncResult
from gevent import monkey
from gevent.pywsgi import WSGIServer

import mushroom
from mushroom import rpc


def setUpModule():
    logging.basicConfig()


class MockTransport(object):
    rpc_engine = None

    def __init__(self):
        self.messages = []

    def send(self, message):
        self.messages.append(message)


class TestService(object):

    def __init__(self):
        self.recv_notify_ok = AsyncResult()
        self.recv_request_ok = AsyncResult()

    def rpc_recv_notify(self, request):
        self.recv_notify_ok.set(True)

    def rpc_recv_request(self, request):
        self.recv_request_ok.set(True)
        return 'recv_request_ok'

    def rpc_recv_error(self, request):
        self.recv_error_ok.set(True)
        return 'recv_request_ok'


class EngineTestCase(unittest.TestCase):

    def setUp(self):
        self.transport = MockTransport()
        self.service = TestService()
        self.engine = rpc.Engine(self.transport,
                rpc.MethodDispatcher(self.service))

    def tearDown(self):
        pass

    def test_recv_notify(self):
        self.engine.handle_message(rpc.Notification(
            message_id=1,
            method='recv_notify',
            data=None))
        self.assertTrue(self.service.recv_notify_ok.get(timeout=1))

    def test_recv_request(self):
        request = rpc.Request(message_id=1, method='recv_request', data=None)
        self.engine.handle_message(request)
        gevent.sleep()
        self.assertEqual(len(self.transport.messages), 1)
        response = self.transport.messages[0]
        self.assertIsInstance(response, rpc.Response)
        self.assertEqual(response.data, 'recv_request_ok')
        self.assertEqual(response.request_message_id, request.message_id)
        self.assertTrue(self.service.recv_request_ok.get(timeout=1))

    def test_recv_response(self):
        request = rpc.Request(message_id=1,
                method='recv_response',
                data=None)
        self.engine.requests[1] = request
        response = rpc.Response(message_id=1,
                request_message_id=request.message_id,
                data='recv_response_ok')
        self.engine.handle_message(response)
        self.assertIs(request.get_response(timeout=1), response)

    def test_recv_error(self):
        request = rpc.Request(message_id=1,
                method='recv_error',
                data=None)
        self.engine.requests[1] = request
        error = rpc.Error(message_id=1,
                request_message_id=request.message_id,
                data='recv_error_ok')
        self.engine.handle_message(error)
        self.assertIs(request.get_response(timeout=1), error)

    def test_send_notify(self):
        # FIXME implement
        self.skipTest('test incomplete')

    def test_send_request(self):
        # FIXME implement
        self.skipTest('test incomplete')

    def test_send_response(self):
        # FIXME implement
        self.skipTest('test incomplete')

    def test_send_error(self):
        # FIXME implement
        self.skipTest('test incomplete')



from mushroom.application import Application
from mushroom.rpc import MethodDispatcher
from mushroom.rpc import Heartbeat
from mushroom.rpc import Notification
from mushroom.rpc import Request
from mushroom.rpc import Response
from mushroom.rpc import Disconnect
from mushroom.session import SessionHandlerAdapter
from mushroom.server import Server

__version__ = '0.9.12'



# This module "extends" the basic RPC methods. The following * import
# makes it possible to just import "mushroom.django.rpc" and also have
# access to the basic RPC method classes.
from mushroom.rpc.methods import *


class PaginatedModelListMethod(PaginatedListMethod):
    model_class = None

    def get_objects(self):
        return self.model_class.objects.all()

    def get_object_count(self):
        return self.all_objects.count()
